@@ -135,6 +135,33 @@ class DataStorage(object):
         except:
             return 0
 
+    def getPreviousObservedValueForType(self, timestamp, type):
+        cursor = self.con.cursor()
+        cursor.execute('''select cast(value as integer)
+                          from observation
+                          where timestamp <= ?
+                          and type = (select id from observation_type where name = ?)
+                          order by timestamp desc
+                          limit 1''', (timestamp, type))
+        try:
+            return cursor.fetchone()[0]
+        except:
+            return None
+
+    def getNextObservedValueForType(self, timestamp, type):
+        cursor = self.con.cursor()
+        cursor.execute('''select cast(value as integer)
+                          from observation
+                          where timestamp >= ?
+                          and type = (select id from observation_type where name = ?)
+                          order by timestamp asc
+                          limit 1''', (timestamp, type))
+        try:
+            return cursor.fetchone()[0]
+        except:
+            return None
+
+
     def countObservations(self):
         cursor = self.con.cursor()
         cursor.execute('select count(id) from observation')
