@@ -28,8 +28,10 @@ import datetime
 import gobject
 
 import beye
+import beye.dataprocessing
 
 import dbus
+
 from dbus.mainloop.glib import DBusGMainLoop
 DBusGMainLoop(set_as_default=True)
 
@@ -143,23 +145,20 @@ class MainWindow(hildon.StackableWindow):
 
         self.menu.append(self.zoomButton)
 
-        self.deleteButton = hildon.Button(gtk.HILDON_SIZE_AUTO, hildon.BUTTON_ARRANGEMENT_HORIZONTAL)
-        self.deleteButton.set_label('Delete data...')
-        self.deleteButton.connect("clicked", self.deleteData)
-
-        self.menu.append(self.deleteButton)
+        self.statisticsButton = hildon.Button(gtk.HILDON_SIZE_AUTO, hildon.BUTTON_ARRANGEMENT_HORIZONTAL)
+        self.statisticsButton.set_label('Statistics')
+        self.statisticsButton.connect("clicked", self.showStatistics)
+        self.menu.append(self.statisticsButton)
 
         self.settingsButton = hildon.Button(gtk.HILDON_SIZE_AUTO, hildon.BUTTON_ARRANGEMENT_HORIZONTAL)
         self.settingsButton.set_label('Settings')
         self.settingsButton.connect("clicked", self.showSettings)
-
         self.menu.append(self.settingsButton)
 
-        #self.incUpd = hildon.Button(gtk.HILDON_SIZE_AUTO, hildon.BUTTON_ARRANGEMENT_HORIZONTAL)
-        #self.incUpd.set_label('Incremental upd.')
-        #self.incUpd.connect("clicked", lambda x: self.graph.updateIncremental())
-
-        #self.menu.append(self.incUpd)
+        self.deleteButton = hildon.Button(gtk.HILDON_SIZE_AUTO, hildon.BUTTON_ARRANGEMENT_HORIZONTAL)
+        self.deleteButton.set_label('Delete data...')
+        self.deleteButton.connect("clicked", self.deleteData)
+        self.menu.append(self.deleteButton)
 
         self.aboutButton = hildon.Button(gtk.HILDON_SIZE_AUTO, hildon.BUTTON_ARRANGEMENT_HORIZONTAL)
         self.aboutButton.set_label('About')
@@ -170,6 +169,10 @@ class MainWindow(hildon.StackableWindow):
         self.menu.show_all()
         self.set_app_menu(self.menu)
 
+    def showStatistics(self, widget):
+        import statistics
+        statistics.Statistics(self.dataStorage)
+
     def showSettings(self, widget):
         dlg = SettingsDialog(self)
         response = dlg.showSettings(self)
@@ -719,71 +722,7 @@ class Graph(hildon.PannableArea):
         self.area.queue_draw_area(area.x, area.y, area.width, area.height)
 
     def getChargeInfo(self, intervalStart, intervalEnd):
-        rawData = self.dataStorage.getObservations((beye.DataSourceHal.CHARGING,
-                                                    beye.DataSourceHal.DISCHARGING,
-                                                    beye.DataSourceInternal.GRAPH_BREAK),
-                                                   intervalStart, intervalEnd)
-        if len(rawData) == 0:
-            return []
-        segment = []
-        segments = [segment]
-        for point in rawData:
-            segment.append(point)
-            if point[2] == beye.DataSourceInternal.GRAPH_BREAK:
-                segment = []
-                segments.append(segment)
-
-        # Beware: while this assumption is close enough of
-        # truth for now, it might not always be.
-        now = intervalEnd
-
-        # Some serious list processing voodoo follows.
-
-        statuses = [] #[[((charging, discharging), start, end), (...), ...], [...], ...]
-        for segment in (x for x in segments if len(x) > 0):
-            intervalStart = segment[0][0]
-            if segment[-1][2] == beye.DataSourceInternal.GRAPH_BREAK:
-                intervalEnd = self.dataStorage.getPreviousObservationTime(segment[-1][0])
-            else:
-                intervalEnd = self.dataStorage.getPreviousObservationTime(now)
-            if intervalEnd == None or intervalStart >= intervalEnd:
-                continue
-
-            statusList = []
-            lastStatus = ((False, True), intervalStart, intervalStart)
-            for point in segment:
-                if point[2] == beye.DataSourceHal.CHARGING:
-                    if point[1] != lastStatus[0][0]:
-                        if point[0] > lastStatus[2]:
-                            statusList.append((lastStatus[0], lastStatus[1], point[0]))
-                        lastStatus = ((bool(point[1]), lastStatus[0][1]), point[0], point[0])
-                    else:
-                        lastStatus = (lastStatus[0], lastStatus[1], point[0])
-                elif point[2] == beye.DataSourceHal.DISCHARGING:
-                    if point[1] != lastStatus[0][1]:
-                        if point[0] > lastStatus[2]:
-                            statusList.append((lastStatus[0], lastStatus[1], point[0]))
-                        lastStatus = ((lastStatus[0][0], bool(point[1])), point[0], point[0])
-                    else:
-                        lastStatus = (lastStatus[0], lastStatus[1], point[0])
-            statusList.append((lastStatus[0], lastStatus[1], intervalEnd))
-            statuses.append([x for x in statusList if x[1] < x[2]])
-        return [x for x in statuses if len(x) > 0]
-
-#        for segment in statuses:
-#            print ""
-#            for status in segment:
-#                if status[0] == (True, False):
-#                    descr = "Charging"
-#                elif status[0] == (True, True):
-#                    descr = "Full"
-#                elif status[0] == (False, True):
-#                    descr = "Discharging"
-#                else:
-#                    descr = "W00t? No battery?"
-#                print "%s (%d minutes) %s - %s" % (descr, (status[2] - status[1]) / 60,
-#                                                   datetime.datetime.fromtimestamp(status[1], beye.utc).astimezone(beye.local),
-#                                                   datetime.datetime.fromtimestamp(status[2], beye.utc).astimezone(beye.local))
+        return beye.dataprocessing.getChargeInfo(self.dataStorage, intervalStart, intervalEnd, intervalEnd)
 
 class SettingsItem(object):
     def __init__(self, *args):
