@@ -29,10 +29,11 @@ if __name__ == "__main__":
         
     
 import beye
+import beye.Daemon as Daemon
 
 if __name__ == "__main__":
     dbFile = beye.setupAndGetDbPath()
     ds = [beye.DataSourceInternal(), beye.DataSourceHal()]
     storage = beye.DataStorage(dbFile, ds)
-    daemon = beye.Daemon(storage, ds)
+    daemon = beye.Daemon.Daemon(storage, ds)
     daemon.run()
